//
//  SettingsController.m
//  MacTrek
//
//  Created by Aqua on 26/05/2006.
//  Copyright 2006 Luky Soft. All rights reserved.
//

#import "SettingsController.h"


@implementation SettingsController

- (void) awakeFromNib {
    [notificationCenter addObserver:self selector:@selector(saveSettings) name:@"MC_LEAVING_SETTINGS"];
    [self setPreviousValues];
}

- (void) setPreviousValues {
    LLPersistantSettings *settings = [LLPersistantSettings defaultSettings];
    
    NSNumber *val;
    
    val = [settings valueForKey:@"MUSIC_LEVEL"];
    if (val != nil) {
        [musicLevel setFloatValue:[val floatValue]];
    }
    val = [settings valueForKey:@"FX_LEVEL"];
    if (val != nil) {
        [fxLevel setFloatValue:[val floatValue]];
    }
    val = [settings valueForKey:@"VOICE_OVER"];
    if (val != nil) {
        if ([val boolValue]) {
           [voiceOver setState:NSOnState];   
        } else {
            [voiceOver setState:NSOffState];
        }        
    }
	val = [settings valueForKey:@"ACCELERATE"];
    if (val != nil) {
        if ([val boolValue]) {
			[accelerateButton setState:NSOnState];   
        } else {
            [accelerateButton setState:NSOffState];
        }        
    }
    val = [settings valueForKey:@"THEME"];
    if (val != nil) {
        [graphicsModel setSelectedSegment:[val intValue]];
    }
}

- (void) saveSettings {
    
    LLPersistantSettings *settings = [LLPersistantSettings defaultSettings];
    
    [settings setLazyValue:[NSNumber numberWithFloat:[self musicLevel]] forKey:@"MUSIC_LEVEL"];
    [settings setLazyValue:[NSNumber numberWithFloat:[self fxLevel]] forKey:@"FX_LEVEL"];
    [settings setLazyValue:[NSNumber numberWithBool:[self voiceEnabled]] forKey:@"VOICE_OVER"];
	[settings setLazyValue:[NSNumber numberWithBool:[self accelerate]] forKey:@"ACCELERATE"];
    [settings setLazyValue:[NSNumber numberWithInt:[self graphicsModel]] forKey:@"THEME"];    
    [settings update];
}

- (float)  musicLevel {
    return [musicLevel floatValue];
}

- (float)  fxLevel {
    return [fxLevel floatValue];
}

- (bool) soundEnabled {
    if (([musicLevel intValue] > 0) ||  ([fxLevel intValue] > 0)) {
        return YES;
    } else {
        return NO;
    }
}

- (bool) accelerate {
    return ([accelerateButton state] == NSOnState);
}

- (bool) voiceEnabled {
    return ([voiceOver state] == NSOnState);
}

- (int)  graphicsModel {
    return [graphicsModel selectedSegment];
}

@end
